#version 330 core
/*
    Voxelverse

    Copyright (c) 2022, Voxelverse Team.
    All rights reserved.
        
    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        
    Redistributions of source code must retain the above copyright notice, this
    list of conditions and the following disclaimer.
    
    Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
    
    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
    AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
    IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
    SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

uniform sampler2D uNormalSampler;
uniform sampler2D uPosSampler;
uniform sampler2D uLightSampler;
uniform sampler2D uSpecularSampler;
uniform vec3 uPlayerPos;
uniform vec3 uAmbientColor;
uniform float uSolarAngle;

in vec2 passTexCoords;

layout (location = 0) out vec4 outColor;

float computeDirectional(vec3 pos, vec3 normal, vec3 lightDir, float specular)
{
    const float shininess = 256.0;

    vec3 viewDir = normalize(uPlayerPos - pos);
    vec3 reflectDir = reflect(-normalize(lightDir), normal);

    return max(0.0, dot(normal, normalize(lightDir)))
        + specular * pow(max(dot(viewDir, reflectDir), 0.0), shininess);
}

void main()
{
    vec3 pos = texture(uPosSampler, passTexCoords).xyz;
    vec3 normal = texture(uNormalSampler, passTexCoords).xyz;
    float specular = texture(uSpecularSampler, passTexCoords).x;

    vec3 color = uAmbientColor * min(1.0, (
        max(0.1, computeDirectional(pos, normal, vec3(cos(uSolarAngle), 0.0, sin(uSolarAngle)), specular))
    ));

    color = max(color, vec3(0.0, 0.0, 0.02));
    
    float distFromSunlight = max(0.0, -pos.z-16.0);
    if (distFromSunlight > 0.0)
    {
        float factor = min(1.0, 1.0 / (distFromSunlight * distFromSunlight));
        color *= factor;
    };

    outColor = vec4(color, 1.0) + texture(uLightSampler, passTexCoords);
}